#include <stdio.h>
#include <math.h>

#define sign(x) real(((x)>0)? 1 : (((x)<0) ? -1 : 0))
#define max(x,y) (((x)>(y)) ? (x):(y));

void TDMA(const Array<real> &al,const Array<real> &am,
          const Array<real> &au,const Array<real> &u,
          const Array<real> &b)
   {
   const int im=u.n_elements();
   Matrix<real> amnew(im),bnew(im);
   int i;
   
   amnew(1)=am(1); bnew(1)=b(1);
   
   for(i=2;i<=im;i++)
      {
      amnew(i) = am(i)-al(i)*  au(i-1)/amnew(i-1);
      bnew(i)  =  b(i)-al(i)*bnew(i-1)/amnew(i-1);
      }
   
   u(im) = bnew(im)/amnew(im);
   
   for(i=im-1;i>=1;i--)
      u(i) = (bnew(i)-au(i)*u(i+1))/amnew(i);
   
   }


void SOR2D(const Matrix<real> &aw, const Matrix<real> &ae,
           const Matrix<real> &as, const Matrix<real> &an,
           const Matrix<real> &ap,       Matrix<real> &u, 
           const Matrix<real> &q,        Matrix<real> &resi,
           real relax)
   {
   const int im=u.n_columns(), jm=u.n_rows();
   int i,j;
      
   for(i=2;i<im;i++)
      for(j=2;j<jm;j++)
         {
         resi(i,j) =     q(i,j)            - ap(i,j)*u(i  ,j  )
                      - aw(i,j)*u(i-1,j  ) - ae(i,j)*u(i+1,j  )
                      - as(i,j)*u(i  ,j-1) - an(i,j)*u(i  ,j+1);
                       
         u(i,j) += relax*resi(i,j)/ap(i,j);              
         }  
   }                 
                     
real L1Norm(const Array<real> &a)
   {
   int i;
   real L1=0.;
   
   for(i=1;i<=a.n_elements();i++)
      L1+=fabs(a(i));
   
   return L1/real(a.n_elements());
   }            
         
real RMS(const Array<real> &a)
   {
   int i;
   real rms=0.;
   
   for(i=1;i<=a.n_elements();i++)
      rms+=a(i)*a(i);
   
   return sqrt(rms/real(a.n_elements()));
   }            
         
void TFInter(Matrix<real> &x)
   {
   const int im=x.n_columns(),jm=x.n_rows();
   int i,j;
   real a,b,t,L0i,L1i,L0j,L1j;
   
   for(i=2;i<im;i++)
      {
      L0i=real(i-im)/real(1-im);
      L1i=real(i-1)/real(im-1);
      for(j=2;j<jm;j++)    
         {
         L0j=real(j-jm)/real(1-jm);
         L1j=real(j-1)/real(jm-1);
         
         a = L0i * x(1,j) + L1i * x(im,j );
         b = L0j * x(i,1) + L1j * x(i ,jm);
         
         t = L0i*L0j*x(1, 1)+L1i*L1j*x(im,jm)
            +L0i*L1j*x(1,jm)+L1i*L0j*x(im, 1);
         
         x(i,j)=a+b-t;
         }
      }
   }         



void OGrid_SOR(Matrix<real> &x, Matrix<real> &y, real eps, real relax,
               Matrix<real> &xi, Matrix<real> &eta)
   {
   const int im=x.n_columns(), jm=x.n_rows();
   int n,i,j,k,ic,jc;
   int iw,ie,js,jn;
   real alpha,beta,gamma,delta,ap,err,
          xw,xe,xs,xn,xnw,xsw,xne,xse,
          yw,ye,ys,yn,ynw,ysw,yne,yse;
   Matrix<real> resi(im,jm),p(im,jm),q(im,jm);       
   Matrix<real> X(im,jm),Y(im,jm);
   
   
   printf("\n iteration    rms(residual)\n\n");
   
   resi=p=q=0.;
   
   for(j=1;j<=jm;j++)
      for(i=1;i<=im;i++)
         {
         if(xi.n_elements()>1)
            for(k=1;k<=xi.n_columns();k++)
               {
               ic=int(xi(k,1)+0.5);
               p(i,j)-=xi(k,2)*sign(i-ic)*exp(-xi(k,3)*fabs(i-ic));
               }
         if(eta.n_elements()>1)      
            for(k=1;k<=eta.n_columns();k++)
               {   
               jc=int(eta(k,1)+0.5);
               q(i,j)-=eta(k,2)*sign(j-jc)*exp(-eta(k,3)*fabs(j-jc));
               }
         }
   
   n=1;
   
   do {
      for(i=1;i<im;i++)
         {
         iw=(i>1) ? (i-1):(im-1);
         ie=i+1;
         
         for(j=2;j<jm;j++)
            {
            js=j-1; jn=j+1;
            
            xw  = x(iw,j  ); xe = x(ie,j );
            xs  = x(i ,js);  xn = x(i ,jn);
            xnw = x(iw,jn); xne = x(ie,jn);
            xsw = x(iw,js); xse = x(ie,js);

            yw  = y(iw,j );  ye = y(ie,j );
            ys  = y(i ,js);  yn = y(i ,jn);
            ynw = y(iw,jn); yne = y(ie,jn);
            ysw = y(iw,js); yse = y(ie,js);
 
            alpha=0.25*((xn-xs)*(xn-xs)+(yn-ys)*(yn-ys));
            beta =0.25*((xe-xw)*(xn-xs)+(ye-yw)*(yn-ys));
            gamma=0.25*((xe-xw)*(xe-xw)+(ye-yw)*(ye-yw));
            delta=0.25*((xe-xw)*(yn-ys)-(xn-xs)*(ye-yw));
            delta*=delta;
 
            ap=2.*(alpha+gamma);
 
            resi(i,j) = alpha*(xw+xe)+gamma*(xs+xn)
                       -0.5*beta*(xne-xnw-xse+xsw)
                       +0.5*delta*(p(i,j)*(xe-xw)+q(i,j)*(xn-xs))
                       -ap*x(i,j);
 
            x(i,j) += relax*resi(i,j)/ap;
 
            resi(i,j) = alpha*(yw+ye)+gamma*(ys+yn)
                       -0.5*beta*(yne-ynw-yse+ysw)
                       +0.5*delta*(p(i,j)*(ye-yw)+q(i,j)*(yn-ys))
                       -ap*y(i,j);
 
            y(i,j) += relax*resi(i,j)/ap;
            
            }
         
         if(i==1)
            for(j=2;j<jm;j++)
               {
               x(im,j) = x(1,j);
               y(im,j) = y(1,j);
               }
         }
      
      err=RMS(resi);
      n++;
      
      if(!(n%50))
         printf("%6d      %10.3g\n",n,err);
         
      }while(err>eps);   
   }                 
   

            






