#include <fstream.h>
#include <stdio.h>
#include <math.h>
#include "settings"
#include "array.h"
#include "cfd.h"

void main()
   {
   double phi,R=10,pi=4.*atan(1.),relax;
   
   // number of gridpoints in i- and j-direction
   
   int im=129,jm=65;
   int i,j;
   
   Matrix<real> X(im,jm),Y(im,jm),XI(2,3),ETA(1,3);
   
   ofstream out("naca.plt");
   
   // airfoil-contour
   
   ifstream contour("naca0012.xy");
   
   // distributing points along outer boundary (circle)

   for(i=1;i<=im;i++)
      {
      phi=double(i-1)/double(im-1);
      phi=2.*pi*phi+(2./3.*phi-2.*phi*phi+4./3.*phi*phi*phi)*(-8.7);
      X(i,jm)=R*cos(phi)+0.5;
      Y(i,jm)=-R*sin(phi);
      contour>>X(i,1)>>Y(i,1);
      }

   // linear distribution along periodic boundary (downstream of airfoil)

   X.col(1).linear(X(1,1),X(1,jm));
   X.col(im)=X.col(1);
   
   // creating initial grid by interpolation
   
   TFInter(X);
   TFInter(Y);
   
   // specifying source-terms to cluster lines near airfoil surface ...
   
   ETA(1,1)=1; ETA(1,2)=150.; ETA(1,3)=.12;
   
   // and near the wake downstream of the airfoil
   
   XI(1,1)=1;  XI(1,2)=50.; XI(1,3)=.3; 
   XI(2,1)=im; XI(2,2)=50.; XI(2,3)=.3; 

   // relaxation-factor 0 < relax < 2 

   cout<<"relaxation factor (0<relax<2): "; cin>>relax;
   
   // generating Grid
   
   OGrid_SOR(X,Y,1e-8,relax,XI,ETA);
       
   // output to TECPLOT-file    
       
   out<<"ZONE I="<<im<<", J="<<jm<<", F=POINT\n"; 
   
   for(j=1;j<=jm;j++)
      for(i=1;i<=im;i++)
         out<<X(i,j)<<"   "<<Y(i,j)<<"\n";

   }
         
